/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2008, Trinity College Computing Center
 * Written by David Chappell
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \brief Applications to decline words according to current language 
 *
 * \author David Chappell <David.Chappell@trincoll.edu>
 * 
 * \ingroup applications
 */
 
/*** MODULEINFO
	<defaultenabled>no</defaultenabled>
 ***/

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 40722 $")

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "asterisk/file.h"
#include "asterisk/logger.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/lock.h"
#include "asterisk/app.h"
#include "asterisk/say.h"

static int app_saycountednoun(struct ast_channel *chan, void *data)
{
	char *parse;
	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(number);
		AST_APP_ARG(noun);
	);

	if (ast_strlen_zero(data)) {
		ast_log(LOG_WARNING, "SayCountedNoun requires two arguments (number|noun)\n");
		return -1;
	}

	parse = ast_strdupa(data);
	AST_STANDARD_APP_ARGS(args, parse);

	if (args.argc != 2) {
		ast_log(LOG_WARNING, "SayCountedNoun requires two arguments\n");
		return -1;
	}

	return ast_say_counted_noun(chan, atoi(args.number), args.noun);
}

static int app_saycountedadj(struct ast_channel *chan, void *data)
{
	char *parse;
	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(number);
		AST_APP_ARG(adjective);
		AST_APP_ARG(gender);
	);

	if (ast_strlen_zero(data)) {
		ast_log(LOG_WARNING, "SayCountedAdj requires two or three arguments (number|adjective|[gender])\n");
		return -1;
	}

	parse = ast_strdupa(data);
	AST_STANDARD_APP_ARGS(args, parse);

	if (args.argc < 2) {
		ast_log(LOG_WARNING, "SayCountedAdj requires at least two arguments\n");
		return -1;
	}

	if (!ast_strlen_zero(args.gender)) {
		if ( strcasecmp(args.gender, "f") && strcasecmp(args.gender,"m") &&
			strcasecmp(args.gender, "c") && strcasecmp(args.gender, "n") ) {
			ast_log(LOG_WARNING, "SayCountedAdj gender option is either 'f', 'm', 'c' or 'n'\n");
			return -1;
		}
	}

	return ast_say_counted_adjective(chan, atoi(args.number), args.adjective, args.gender);
}

static struct application_list_item {
	char name[AST_MAX_APP];
	int (*execute)(struct ast_channel *chan, void *data);
	char *synopsis;
	char *description;
} apps[] =
{
	{ "SayCountedNoun", app_saycountednoun,
	"Say a noun in counting form",
	"  SayCountedNoun(number,filename): When saying how many there are of an\n"
	"object, select and play the proper form of the noun that is its name.\n"
	"The various singular and plural forms are selected by adding suffixes\n"
	"to filename.\n"
	"\n"
	"number - the number of things\n"
	"filename - sound file name stem\n"
	"\n"
	"If the channel language is English, then the suffix will be either\n"
	"empty or \"s\".\n"
	"\n"
	"If the channel language is Russian or some other slavic language,\n"
	"then the suffix will be empty for nominative, \"x1\" for genative\n"
	"singular, and \"x2\" for genative plural.\n"
	},

	{ "SayCountedAdj", app_saycountedadj,
	"Say an adjective in counting form",
	"  SayCountedAdj(number,filename,gender): When saying how many there are of an\n"
	"object, select and play the proper form of an adjective which forms part of\n"
	"its name. The various singular and plural forms are selected by adding suffixes\n"
	"to filename.\n"
	"\n"
	"number - the number of things\n"
	"soundfile - sound file name stem\n"
	"gender - one of m, f, n, or c\n"
	"\n"
	"If the channel language is English, then no suffix will ever be added.\n"
	"\n"
	"If the channel language is Russian or some other slavic language,\n"
	"then the suffix will the gender argument for nominative, and \"x2\" for\n"
	"genative plural.\n"
	}
};

static int load_module(void)
{
	int x, res = 0;
	for (x=0; x<sizeof(apps) / sizeof(struct application_list_item); x++) {
		res |= ast_register_application(apps[x].name, apps[x].execute, apps[x].synopsis, apps[x].description);
	}
	return res;
}

static int unload_module(void)
{
	int x, res = 0;
	for (x=0; x<sizeof(apps) / sizeof(struct application_list_item); x++) {
		res |= ast_unregister_application(apps[x].name);
	}
	return res;
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Decline words according to channel language");
